@@ -39,6 +39,10 @@ def read(file_descript):
 def write(file_descript, motifs, alphabet=None, background=None, mode="w"):
 
     motifs = [motifs] if not isinstance(motifs, (list, tuple, pd.Series)) else motifs
+
+    if len(motifs) == 0:
+        raise RuntimeError("Unable to write motif file with zero motifs")
+
     alphabet = alphabet if alphabet is not None else motifs[0].alphabet
     background = np.array([[1 / len(alphabet)] * len(alphabet)]) if background is None else background
 
